#!/usr/bin/env python

"Resources for use with WebStack."

import WebStack.Generic
import XSLForms.Fields

class XSLFormsResource:

    "A generic XSLForms resource for use with WebStack."

    path_encoding = "iso-8859-1"
    encoding = "utf-8"

    def get_fields_from_body(self, trans, encoding):

        """
        From the given transaction 'trans' and using the stated text 'encoding'
        get the field values from the request body and return a dictionary
        mapping field names to lists of such values.
        """

        text = trans.get_request_stream().read().decode(encoding)
        parameters = {}
        for text_line in text.split("\r\n"):
            text_parts = text_line.split("=")
            text_name, text_value = text_parts[0], "=".join(text_parts[1:])
            if not parameters.has_key(text_name):
                parameters[text_name] = []
            # NOTE: Workaround from posted text.
            if text_value[-1] == "\x00":
                text_value = text_value[:-1]
            parameters[text_name].append(text_value)
        return parameters

    def respond(self, trans):

        """
        Respond to the request described by the given transaction 'trans'.
        """

        global XSLForms # NOTE: Strangely required to avoid UnboundLocalError!

        # Only obtain field information according to the stated method.

        method = trans.get_request_method()
        path_info = trans.get_path_info()

        # Handle typical request methods, processing request information.

        if method == "GET":

            # Get the fields from the request path (URL).

            form = XSLForms.Fields.Form(encoding=self.path_encoding, values_are_lists=1)
            parameters = trans.get_fields_from_path()
            form.set_parameters(parameters)

        elif method == "POST":

            # Get the fields from the request body.

            form = XSLForms.Fields.Form(encoding=self.encoding, values_are_lists=1)

            # Handle requests for in-page updates.

            if path_info in self.in_page_resources.keys():
                parameters = self.get_fields_from_body(trans, self.encoding)
            else:
                parameters = trans.get_fields_from_body(self.encoding)

            # Get the XML representation of the request.

            form.set_parameters(parameters)

        else:

            # Initialise empty containers.

            parameters = {}
            documents = {}

        # Call an overridden method with the processed request information.

        self.respond_to_form(trans, form)

    def respond_to_form(self, trans, form):

        """
        Respond to the request described by the given transaction 'trans', using
        the given 'form' object to conveniently retrieve field (request
        parameter) information and structured form information (as DOM-style XML
        documents).
        """

        trans.set_response_code(500)
        raise WebStack.Generic.EndOfResponse

# vim: tabstop=4 expandtab shiftwidth=4
